@@ -113,7 +113,7 @@ where
             (Some(t), _) if !t.is_expired() && !force_refresh => {
                 // unexpired token found
                 log::debug!("found valid token in cache: {:?}", t);
-                Ok(t.into())
+                Ok(t)
             }
             (
                 Some(TokenInfo {
@@ -123,12 +123,21 @@ where
                 Some(app_secret),
             ) => {
                 // token is expired but has a refresh token.
-                let token_info = RefreshFlow::refresh_token(
+                let token_info_result = RefreshFlow::refresh_token(
                     &self.inner.hyper_client,
                     app_secret,
                     &refresh_token,
                 )
-                .await?;
+                .await;
+                let token_info = if let Ok(token_info) = token_info_result {
+                    token_info
+                } else {
+                    // token refresh failed.
+                    self.inner
+                        .auth_flow
+                        .token(&self.inner.hyper_client, scopes)
+                        .await?
+                };
                 self.inner
                     .storage
                     .set(hashed_scopes, token_info.clone())
@@ -325,14 +334,14 @@ impl ApplicationDefaultCredentialsAuthenticator {
     pub async fn builder(
         opts: ApplicationDefaultCredentialsFlowOpts,
     ) -> ApplicationDefaultCredentialsTypes<DefaultHyperClient> {
-        Self::with_client(DefaultHyperClient, opts).await
+        Self::with_client(opts, DefaultHyperClient).await
     }
 
     /// Use the builder pattern to deduce which model of authenticator should be used and allow providing a hyper client
     #[cfg(feature = "service_account")]
     pub async fn with_client<C>(
-        client: C,
         opts: ApplicationDefaultCredentialsFlowOpts,
+        client: C,
     ) -> ApplicationDefaultCredentialsTypes<C>
     where
         C: HyperClientBuilder,
@@ -727,6 +736,10 @@ pub trait HyperClientBuilder {
 
     /// Create a hyper::Client
     fn build_hyper_client(self) -> hyper::Client<Self::Connector>;
+
+    /// Create a `hyper::Client` for tests (HTTPS not required)
+    #[doc(hidden)]
+    fn build_test_hyper_client(self) -> hyper::Client<Self::Connector>;
 }
 
 #[cfg(feature = "hyper-rustls")]
@@ -768,7 +781,28 @@ impl HyperClientBuilder for DefaultHyperClient {
 
     fn build_hyper_client(self) -> hyper::Client<Self::Connector> {
         #[cfg(feature = "hyper-rustls")]
-        let connector = hyper_rustls::HttpsConnector::with_native_roots();
+        let connector = hyper_rustls::HttpsConnectorBuilder::new()
+            .with_native_roots()
+            .https_only()
+            .enable_http1()
+            .enable_http2()
+            .build();
+        #[cfg(all(not(feature = "hyper-rustls"), feature = "hyper-tls"))]
+        let connector = hyper_tls::HttpsConnector::new();
+
+        hyper::Client::builder()
+            .pool_max_idle_per_host(0)
+            .build::<_, hyper::Body>(connector)
+    }
+
+    fn build_test_hyper_client(self) -> hyper::Client<Self::Connector> {
+        #[cfg(feature = "hyper-rustls")]
+        let connector = hyper_rustls::HttpsConnectorBuilder::new()
+            .with_native_roots()
+            .https_or_http()
+            .enable_http1()
+            .enable_http2()
+            .build();
         #[cfg(all(not(feature = "hyper-rustls"), feature = "hyper-tls"))]
         let connector = hyper_tls::HttpsConnector::new();
 
@@ -787,6 +821,10 @@ where
     fn build_hyper_client(self) -> hyper::Client<C> {
         self
     }
+
+    fn build_test_hyper_client(self) -> hyper::Client<Self::Connector> {
+        self
+    }
 }
 
 /// How should the acquired tokens be stored?
